<?php
/**
 * Created by PhpStorm.
 * User: Pascal Roessler
 * Date: 05/22/23
 * Time: 10:45 AM
 */

namespace IMATHUZH\Qfq\Core\Form;

use CodeException;
use DbException;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;
use IMATHUZH\Qfq\Core\Store\Store;
use UserFormException;
use UserReportException;

/**
 * Class FormElementRenderBS3
 * @package qfq
 */
class FormElementRenderBS3
{
    /**
     *
     * Processes the formelement and generates its HTML representation.
     * This static method takes an input object representing a form element, extracts its attributes, and constructs the HTML code
     * for rendering the form element.
     * And any additional buttons or information associated with the form element.
     *
     * Below is an example of the generated HTML.
     *  * <pre>
     * &lt;input id="10544-223667-22-0" name="text2-22" maxlength="55" type="text" value="test" data-hidden="no" data-required="no" class="form-control" data-reference="text2" data-match-error="Fields do not match" data-error="Error"&gt;
     * </pre>
     *
     * @param object $input The input object representing the formElement.
     *
     * @return string The HTML representation of the formElement object.
     *
     * @throws CodeException
     */
    public function renderFormElement(object $input): string
    {
        $attributes = '';

        foreach ($input->htmlAttributes as $key => $value) {
            $attributeValue = is_array($value) ? $value[FE_VALUE] : $value;
            $flagOmitEmpty = is_array($value) && isset($value[FLAG_OMIT_EMPTY]) ? $value[FLAG_OMIT_EMPTY] : null;
            $modeEscape = is_array($value) && isset($value[MODE_ESCAPE]) ? $value[MODE_ESCAPE] : null;
            if(!empty($attributeValue) || $key == FE_VALUE){
                $attributes .= Support::doAttribute($key, $attributeValue, $flagOmitEmpty, $modeEscape);
            }
        }

        if($input->htmlTag == 'textarea'){
            $html = "<$input->htmlTag $attributes>" . $input->textAreaValue . "</$input->htmlTag>";
        } else{
            $html = "<$input->htmlTag $attributes>";
        }

        if (!empty($input->_extraButtonHtml)) {
            if ($input->flagTextarea) {
                $html .= $input->_extraButtonHtml;
            } else {
                $html = Support::wrapTag('<div class="input-group">', $html . $input->_extraButtonHtml);
            }
        }
        if (!empty($input->extraButtonInfo)) {
            $html .= $input->extraButtonInfo;
        }

        $html .= HelperFormElement::getHelpBlock();

        if (!empty($input->characterCountWrap)) {

            $input->attributeCC = Support::doAttribute(FE_ID,$input->attributeCC);
            $input->classCC = Support::doAttribute(FE_CLASS,$input->classCC);

            $input->elementCharacterCount = "<span $input->attributeCC $input->classCC></span>";
            if ($input->characterCountWrap != '') {
                $arr = explode('|', $input->characterCountWrap, 2);
                $arr[] = '';
                $arr[] = ''; //skip check that at least 2 elements exist
                $input->elementCharacterCount = $arr[0] . $input->elementCharacterCount . $arr[1];
            }

        }
        if (!empty($input->elementCharacterCount)) {
            $html .= $input->elementCharacterCount;
        }

        return $html;
    }
    /**
     * Prepare code of 'lock', 'password', 'info' to extend a FormElement.
     * The 'info' will always be added, 'lock' and 'password' only on FE with mode=show|required
     * Depending on $showInside:
     *    * true: a button is shown inside the 'input' or 'select' element.
     *    * false: an icon is shown below the FormElement.
     *
     * 'Add' means:
     *    * $rcButton contains all HTML button code. The calling function is responsible to build the correct code.
     *    * $input->extraButtonInfo has been wrapped with HTML-tag and HTML-id.
     *    * $input->mode - for 'lock' it will be faked to 'readonly'
     *    * $input->type - for 'password' it will be faked to 'password'
     *
     * @param FormElementInput $input
     * @param bool $showInline
     *
     * @throws CodeException|UserReportException|DbException|UserFormException
     */
    public function prepareExtraButton(FormElementInput $input, bool $showInline)
    {

        $store = Store::getInstance();

        $infoSymbolInside = $store->getVar(SYSTEM_EXTRA_BUTTON_INFO_INLINE, STORE_SYSTEM);
        $infoSymbolOutside = $store->getVar(SYSTEM_EXTRA_BUTTON_INFO_BELOW, STORE_SYSTEM);

        if (SYSTEM_EXTRA_BUTTON_INFO_POSITION_BELOW == $store->getVar(SYSTEM_EXTRA_BUTTON_INFO_POSITION, STORE_SYSTEM)) {
            $showInline = false;
        }

        $extraButton = '';
        $id = $input->htmlId;

        if (false !== strpos($input->name, FE_TEMPLATE_GROUP_NAME_PATTERN)) {
            $id .= '-' . FE_TEMPLATE_GROUP_NAME_PATTERN;
        }

        $minWidth = $input->inputExtraButtonMinWidth ?? '';
        $minWidth .= 'px';

        // INFO: $showinline =- TRUE ('input' elements)
        if (!empty($input->extraButtonInfo) && $showInline) {
            $extraButton .= <<<EOF
            <button type="button" class="btn btn-info" onclick="$('#$id-extra-info').slideToggle('swing')">
                $infoSymbolInside
            </button>
EOF;

            $value = $input->extraButtonInfo ;
            $input->extraButtonInfo = <<<EOF
            <div class="alert alert-info" id="$id-extra-info" style="display: none; min-width: $minWidth;">
                <p>$value</p>
            </div>
EOF;
        }

//        <span class="glyphicon glyphicon-info-sign text-info" aria-hidden="true" onclick="$('#$id-extra-info').slideToggle('swing')"></span>

        $js = " onclick=\"$('#$id-extra-info').slideToggle('swing')\" ";
        $arr = explode(' ', $infoSymbolOutside, 2);
        $infoSymbolOutside = $arr[0] . $js . $arr[1];

        // INFO: $showinline == FALSE (e.g. 'textarea' elements)
        if (!empty($input->extraButtonInfo) && !$showInline) {
            $class = $input->extraButtonInfoClass;
            $extraButton .= <<<EOF
            <span class="$class">$infoSymbolOutside</span>
EOF;

            $value = $input->extraButtonInfo;
            $input->extraButtonInfo = <<<EOF
            <div class="alert alert-info" id="$id-extra-info" style="display: none; min-width: $minWidth;">
                <p>$value</p>
            </div>
EOF;
        }

        $skip = (!($input->mode == FE_MODE_SHOW
            || $input->mode == FE_MODE_REQUIRED
            || $input->mode == FE_MODE_SHOW_REQUIRED));

        // LOCK
        if (!$skip && HelperFormElement::booleParameter($input->extraButtonLock ?? '-')) {
            switch ($input->type) {
                case FE_TYPE_SELECT:
                    $extraButton .= <<<EOF
                    <button class="btn btn-info"
                            onclick="$('#$id').toggleClass('qfq-disabled')">
                        <span class="glyphicon glyphicon-lock" aria-hidden="true"></span>
                    </button>
                    <script>$('#$id').addClass('qfq-disabled')</script>
EOF;
                    break;
                case FE_TYPE_CHECKBOX:
                case FE_TYPE_RADIO:
                    $extraButton .= <<<EOF
                    <button class="btn btn-info"
                            onclick="$('[id^=$id]').parent('label').toggleClass('qfq-disabled')">
                        <span class="glyphicon glyphicon-lock" aria-hidden="true"></span>
                    </button>
                    <script>$('[id^=$id]').parent('label').addClass('qfq-disabled')</script>
EOF;
                    break;
                default:
                    $extraButton .= <<<EOF
                    <button class="btn btn-info"
                            onclick="$('#$id').prop('readonly',!$('#$id').prop('readonly'))">
                        <span class="glyphicon glyphicon-lock" aria-hidden="true"></span>
                    </button>
                    <script>$('#$id').prop('readonly', true)</script>
EOF;
                    break;

            }
        }
        // PASSWORD
        if (!$skip && HelperFormElement::booleParameter($input->extraButtonPassword ?? '-')) {

            $input->type = 'password';

            $extraButton .= <<<EOF
            <button class="btn btn-info"
                    onclick="$('#$id').toggleClass('qfq-password')">
                <span class="glyphicon glyphicon-eye-open" aria-hidden="true"></span>
            </button>
EOF;
        }

        $input->_extraButtonHtml = Support::wrapTag('<div class="input-group-btn">', $extraButton, true);

    }

}